#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "QFileDialog"
#include "QMessageBox"
#include "csv.h"
#include "qdebug.h"
#include <math.h>
#include <aplot.h>
#include <qcustomplot.h>
#include <qprogressdialog.h>
#include <QtConcurrent>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);


    ui->_0_cp_rawResHistoGram->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom | QCP::iSelectAxes);
    ui->_4_cp_doseChoiceDistribution->setInteractions(QCP::iRangeDrag | QCP::iRangeZoom | QCP::iSelectAxes);

    connect(ui->_0_cp_rawResHistoGram, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(mousePressDE()));
    connect(ui->_0_cp_rawResHistoGram, SIGNAL(mouseWheel(QWheelEvent*)), this, SLOT(mouseWheelDE()));
    connect(ui->_4_cp_doseChoiceDistribution, SIGNAL(mousePress(QMouseEvent*)), this, SLOT(mousePress_4_cp_doseDist()));
    connect(ui->_4_cp_doseChoiceDistribution, SIGNAL(mouseWheel(QWheelEvent*)), this, SLOT(mouseWheel_4_cp_doseDist()));
    connect(ui->actionSave,SIGNAL(triggered()),this,SLOT(saveModels()));
    connect(ui->actionOpen,SIGNAL(triggered()),this,SLOT(loadModels()));

    ui->_3_cp_epDistribution->setVisible(false);
    ui->_3_lb_indpVar->setVisible(false);
    ui->_3_sl_doseSlider->setVisible(false);
    ui->__mainToolBox->setCurrentIndex(0);
    ui->_1_tw_etaDefinition->setVisible(false);
    ui->_1_lb_etaText->setVisible(false);
    ui->_1_cb_numEtas->setVisible(false);
    ui->_1_cb_iivEndPoint->setVisible(false);
    ui->_1_sb_numSimulations->setVisible(false);

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::saveModels(){

    QString fileName=QFileDialog::getSaveFileName(this,"Save...");

    if (fileName.isEmpty())
             return;
         else {
             QFile file(fileName);
             if (!file.open(QIODevice::WriteOnly)) {
                 QMessageBox::information(this, tr("Unable to open file"),
                     file.errorString());
                 return;
             }

             QDataStream out(&file);
             out.setVersion(QDataStream::Qt_5_2);

             out<<rawResDatas;
             out<<rawResHeaders;
             out<<ui->_1_le_possibleDoses->text();

             QStringList tempQSL;
             QVector<QVector<QVector<double> > > omegaMatricies;
             omegaMatricies.clear();

             QVector<QVector<QVector<int> > > omegaIndicies;
             omegaIndicies.clear();

             for (int i=0;i<ui->_1_tw_endPoints->rowCount()&i<endPoints.size();i++){
                 if(ui->_1_tw_endPoints->item(i,0)!=0){
                     tempQSL<<ui->_1_tw_endPoints->item(i,0)->text();
                 }else{
                     tempQSL<<"";
                 }

                 omegaMatricies<<endPoints.at(i).omegaMatrix;
                 omegaIndicies<<endPoints.at(i).omegaIndecies;
             }
                     out << tempQSL;
                     out << omegaMatricies;
                     out << omegaIndicies;

                     out << ui->_1_cb_iivEndPoint->currentIndex();

                     tempQSL.clear();
             for(int i=0;i<ui->_2_tw_weightDefinition->rowCount()&i<this->endPoints.size();i++){
                 if(ui->_2_tw_weightDefinition->item(i,0)!=0){
                     tempQSL<<ui->_2_tw_weightDefinition->item(i,0)->text();
                 }else{
                     tempQSL<<"";
                 }
             }
             out<<tempQSL;

             out<<ui->_2_cb_logScale->isChecked();

             out << ui->_3_cb_aveORselect->currentIndex();
             out << ui->_3_cb_indOFVone->currentIndex();

             out << ui->_3_le_lowerBound->text();
             out << ui->_3_le_upperBound->text();

             out << this->selectedDoses;
             out << this->dsProbabilitiesSL;
             out << this->dsProbabilities;
             out << this->cash_endpoints;
    }
}

void MainWindow::loadModels(){

    QString fileName=QFileDialog::getOpenFileName(this,"Open...");


    if (fileName.isEmpty())
             return;
         else {

        ui->__lb_directory->setText(fileName);
             QFile file(fileName);

             if (!file.open(QIODevice::ReadOnly)) {
                 QMessageBox::information(this, tr("Unable to open file"),
                     file.errorString());
                 return;
             }

             QDataStream out(&file);
             out.setVersion(QDataStream::Qt_5_2);

             if (!out.atEnd()){
                 rawResDatas.clear();
                 out>>rawResDatas;
             }

             if (!out.atEnd()){
                 rawResHeaders.clear();
                 out>>rawResHeaders;
             }

             int numEstModel=rawResDatas.size();

             if (numEstModel==0){
                 QMessageBox newMessage;
                 newMessage.setText("raw results file cannot be found in this folder, check the namr format definition.");
                 newMessage.exec();
                 ui->__mainToolBox->setCurrentIndex(0);
             }else{
                 ui->__mainToolBox->setVisible(true);
                 ui->_0_sb_model->setMaximum(this->rawResDatas.size()-1);

                 ui->_1_tw_endPoints->setRowCount(this->rawResDatas.size());
                 ui->_0_sb_data->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
                 ui->_0_sb_data->setVisible(true);
                 ui->_2_sb_dataNumber->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
                 ui->_2_sb_dataNumber->setVisible(true);
                 ui->_3_cb_dataNumber->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
                 ui->_3_cb_dataNumber->setVisible(true);
                 ui->_0_lb_dataNumber->setVisible(true);
                 ui->_2_lb_dataNumber->setVisible(true);
                 ui->_3_lb_dataNumber->setVisible(true);

                 this->endPoints.resize(numEstModel);

                 QStringList tempQSL;
                 for (int i=0;i<numEstModel;i++){
                     this->endPoints[i].name="model "+QString::number(i);
                     tempQSL<<this->endPoints.at(i).name;
                     endPoints[i].OFV_index=this->OFV_index;
                     endPoints[i].THETA_index=this->THETA_index;
                     endPoints[i].ETA_index=this->ETA_index;
                 }
                 ui->_1_tw_endPoints->setVerticalHeaderLabels(tempQSL);
                 refresh_0_tw_RawResults();
             }




             QString tempString;
             if (!out.atEnd()){
             out >> tempString;
             if (!tempString.isEmpty())
                ui->_1_le_possibleDoses->setText(tempString);
             }

             QStringList tempQSL;
             QVector<QVector<QVector<double> > > omegaMatricies;
             QVector<QVector<QVector<int> > > omegaIndicies;

             if (!out.atEnd()){
            out >> tempQSL;
            out >> omegaMatricies;
            out >> omegaIndicies;
            if(!tempQSL.isEmpty())
                 for (int i=0;i<ui->_1_tw_endPoints->rowCount()&i<tempQSL.size()&i<endPoints.size();i++){
                     endPoints[i].numSample=0;

                     ui->_1_tw_endPoints->setItem(i,0,new QTableWidgetItem(tempQSL.at(i)));
                     endPoints[i].checkETA();
                     endPoints[i].omegaMatrix=omegaMatricies.at(i);
                     endPoints[i].omegaIndecies=omegaIndicies.at(i);
                     refresh__1_tw_etaDefinition();

                     endPoints[i].numSample=ui->_1_sb_numSimulations->value();
                 }
             }


            int tempI;
            if (!out.atEnd()){
            out>>tempI;
            ui->_1_cb_iivEndPoint->setCurrentIndex(tempI);
            }

            if (!out.atEnd()){
            tempQSL.clear();
            out >> tempQSL;

            ui->_2_tw_weightDefinition->setRowCount(endPoints.size());
            if(!tempQSL.isEmpty())
            for(int i=0;i<ui->_2_tw_weightDefinition->rowCount()&i<tempQSL.size();i++){
                ui->_2_tw_weightDefinition->setItem(i,0,new QTableWidgetItem(tempQSL.at(i)));
            }
            }

            bool tempB;
            if (!out.atEnd()){
            out >> tempB;
            ui->_2_cb_logScale->setChecked(tempB);
            }

            if (!out.atEnd()){
            out >> tempI;
            ui->_3_cb_aveORselect->setCurrentIndex(tempI);
            }
            if (!out.atEnd()){
            out >> tempI;
            ui->_3_cb_indOFVone->setCurrentIndex(tempI);
            }

            QString temoString;

            if (!out.atEnd()){
            out >> tempString;
            ui->_3_le_lowerBound->setText(tempString);
            this->targetLowerbound=tempString.toDouble();
            }
            if (!out.atEnd()){
            out >> tempString;
            ui->_3_le_upperBound->setText(tempString);
            this->targetUpperbound=tempString.toDouble();
            }

            if (!out.atEnd()){
            out >> this->selectedDoses;
            }
            if (!out.atEnd()){
            out >> this->dsProbabilitiesSL;
            csvDisplay(ui->_4_tw_doseChoice,this->selectedDoses,this->dsProbabilitiesSL);
            ui->_4_le_probabilities->setText(dsProbabilitiesSL.join(","));
            }
            if (!out.atEnd()){
            out >> this->dsProbabilities;
            }
            if (!out.atEnd()){
            out >> this->cash_endpoints;
            }
    }
}

void MainWindow::on___pb_import_clicked()
{
    this->rawResHeaders.clear();
    this->rawResDatas.clear();

    QString raw_res_fileDirectory;
    raw_res_fileDirectory=QFileDialog::getExistingDirectory(this, ("Select the directory where PsN bootstrap raw_results files are."));
    ui->__lb_directory->setText(raw_res_fileDirectory);

        int numEstModel=0;
        QFile file(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(numEstModel)+ui->_m1_le_dataNumberPrefex->text()+"0"+ui->_m1_le_dataNumberSuffix->text());
        if (file.exists()){
            while (file.exists()){
                numEstModel=numEstModel+1;
                file.setFileName(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(numEstModel)+ui->_m1_le_dataNumberPrefex->text()+"0"+ui->_m1_le_dataNumberSuffix->text());
            }
            this->rawResDatas.resize(numEstModel);

            QProgressDialog progress("Importing Simulation Data...", "Abort Import", 0, numEstModel, this);
                 progress.setWindowModality(Qt::WindowModal);

            int ii=0;
                int j=0;
                file.setFileName(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(ii)+ui->_m1_le_dataNumberPrefex->text()+"0"+ui->_m1_le_dataNumberSuffix->text());
                this->rawResHeaders<<csvWithHeaderReadHeader(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(ii)+ui->_m1_le_dataNumberPrefex->text()+QString::number(j)+ui->_m1_le_dataNumberSuffix->text());
                while(file.exists()){
                    if(this->rawResHeaders.at(ii)!=csvWithHeaderReadHeader(raw_res_fileDirectory+"/raw_results_estimation_model"+QString::number(ii)+"_data"+QString::number(j)+".csv")){
                        QMessageBox newMessage;
                        newMessage.setText("Headers of the raw_results files do not match.");
                        newMessage.exec();
                        break;
                    }
                    this->rawResDatas[ii]<<csvWithHeaderReadToDouble(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(ii)+ui->_m1_le_dataNumberPrefex->text()+QString::number(j)+ui->_m1_le_dataNumberSuffix->text());

                    j++;
                    file.setFileName(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(ii)+ui->_m1_le_dataNumberPrefex->text()+QString::number(j)+ui->_m1_le_dataNumberSuffix->text());

                }

                progress.setValue(1);

            for (int i=1;i<numEstModel;i++){
                file.setFileName(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(i)+ui->_m1_le_dataNumberPrefex->text()+QString::number(0)+ui->_m1_le_dataNumberSuffix->text());
                this->rawResHeaders<<csvWithHeaderReadHeader(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(i)+ui->_m1_le_dataNumberPrefex->text()+QString::number(0)+ui->_m1_le_dataNumberSuffix->text());

                for(int j=0;j<this->rawResDatas[0].size();j++){
//                    if(file.exists()&this->rawResHeaders.at(i)!=csvWithHeaderReadHeader(raw_res_fileDirectory+"/raw_results_estimation_model"+QString::number(i)+"_data"+QString::number(j)+".csv")){
//                        QMessageBox newMessage;
//                        newMessage.setText("Headers of the raw_results files do not match.");
//                        newMessage.exec();
//                        break;
//                    }else

                    if(!file.exists()){
                        if (j>0){
                        this->rawResDatas[i]<<this->rawResDatas[i][j-1];
                        for (int k=0;k<this->rawResDatas[i][j].size();k++)
                            this->rawResDatas[i][j][k].fill(0.0/0.0);
                        }else{
                            QMessageBox newMessage;
                            newMessage.setText("raw_results file import failed.");
                            newMessage.exec();
                            break;
                        }
                    }else{
                        this->rawResDatas[i]<<csvWithHeaderReadToDouble(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(i)+ui->_m1_le_dataNumberPrefex->text()+QString::number(j)+ui->_m1_le_dataNumberSuffix->text());
                    }
                    file.setFileName(raw_res_fileDirectory+"/raw_results_"+ui->_m1_le_modelNumPrefex->text()+QString::number(i)+ui->_m1_le_dataNumberPrefex->text()+QString::number(j+1)+ui->_m1_le_dataNumberSuffix->text());

                }
                progress.setValue(i);
                if (progress.wasCanceled())
                    break;
            }

        }

        if (numEstModel==0){
            QMessageBox newMessage;
            newMessage.setText("raw results file cannot be found in this folder, check the namr format definition.");
            newMessage.exec();
            ui->__mainToolBox->setCurrentIndex(0);
        }else{
            ui->__mainToolBox->setVisible(true);
            ui->_0_sb_model->setMaximum(this->rawResDatas.size()-1);

            ui->_1_tw_endPoints->setRowCount(this->rawResDatas.size());
            ui->_2_tw_weightDefinition->setRowCount(this->rawResDatas.size());
            ui->_2_tw_relativeWeight->setColumnCount(this->rawResDatas.size());
            ui->_0_sb_data->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
            ui->_0_sb_data->setVisible(true);
            ui->_2_sb_dataNumber->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
            ui->_2_sb_dataNumber->setVisible(true);
            ui->_3_cb_dataNumber->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
            ui->_3_cb_dataNumber->setVisible(true);
            ui->_0_lb_dataNumber->setVisible(true);
            ui->_2_lb_dataNumber->setVisible(true);
            ui->_3_lb_dataNumber->setVisible(true);

            this->endPoints.resize(numEstModel);

            QStringList tempQSL;
            for (int i=0;i<numEstModel;i++){
                this->endPoints[i].name="model "+QString::number(i);
                tempQSL<<this->endPoints.at(i).name;
                endPoints[i].OFV_index=this->OFV_index;
                endPoints[i].THETA_index=this->THETA_index;
                endPoints[i].ETA_index=this->ETA_index;
            }
            ui->_1_tw_endPoints->setVerticalHeaderLabels(tempQSL);
            ui->_2_tw_weightDefinition->setVerticalHeaderLabels(tempQSL);
            ui->_2_tw_relativeWeight->setHorizontalHeaderLabels(tempQSL);
            refresh_0_tw_RawResults();
        }

        file.close();



}

void MainWindow::on__0_sb_model_valueChanged(int arg1)
{
    ui->_0_sb_data->setMaximum(this->rawResDatas.at(ui->_0_sb_model->value()).size()-1);
    refresh_0_tw_RawResults();
    refresh_0_cp_rawResHistoGram();
}

void MainWindow::refresh_0_tw_RawResults(){
    int dataNumber=ui->_0_sb_data->value();
    if (dataNumber<0)
        dataNumber=0;

    if (ui->_0_sb_model->value()<rawResDatas.size()&ui->_0_sb_model->value()>=0)
        if(rawResDatas.at(ui->_0_sb_model->value()).size()>dataNumber&dataNumber>=0)
            if(rawResHeaders.size()>ui->_0_sb_model->value()&ui->_0_sb_model->value()>=0)
                csvDisplay(ui->_0_tw_RawResults,this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber),this->rawResHeaders.at(ui->_0_sb_model->value()));

}

void MainWindow::on__0_sb_data_valueChanged(int arg1)
{
    refresh_0_tw_RawResults();
    refresh_0_cp_rawResHistoGram();
}

void MainWindow::refresh_2_tw_weightDefinition(){
    if (this->rawResDatas.size()>0){

    ui->_2_tw_weightDefinition->setRowCount(this->rawResHeaders.size());
    ui->_2_tw_relativeWeight->setColumnCount(this->rawResHeaders.size());
    ui->_2_tw_relativeWeight->setRowCount(this->rawResDatas.at(0).at(ui->_2_sb_dataNumber->value()).size());
    QStringList tempQSL;

    for (int i=0;i<this->rawResHeaders.size();i++){
        ui->_2_tw_weightDefinition->setItem(i,0,new QTableWidgetItem("(0-"+QString::number(this->endPoints[i].epParser.extract_variables().size()-3)+"-ofv/2)"));
        tempQSL<<this->endPoints.at(i).name;
    }

    ui->_2_tw_weightDefinition->setVerticalHeaderLabels(tempQSL);
    ui->_2_tw_relativeWeight->setHorizontalHeaderLabels(tempQSL);
    refresh_2_tw_relativeWeight();
    }

}

void MainWindow::refresh_2_tw_relativeWeight(){

    csvDisplay(ui->_2_tw_relativeWeight,calculate_relativeWeight(ui->_2_sb_dataNumber->value()));

    QVector<int> choice=make_ModelSelection(ui->_2_sb_dataNumber->value());

    for (int i=0;i<choice.size();i++){
        for (int j=0;j<ui->_2_tw_relativeWeight->columnCount();j++){
            if (ui->_2_tw_relativeWeight->item(i,j)!=0){
                ui->_2_tw_relativeWeight->item(i,j)->setBackgroundColor(Qt::white);
                ui->_2_tw_relativeWeight->item(i,j)->setTextColor(Qt::black);
            }
        }
        if (choice.at(i)>=0)
            if (ui->_2_tw_relativeWeight->item(i,choice.at(i))!=0){
                ui->_2_tw_relativeWeight->item(i,choice.at(i))->setBackgroundColor(Qt::red);
                ui->_2_tw_relativeWeight->item(i,choice.at(i))->setTextColor(Qt::white);
            }


    }

}

QVector<QVector<double> > MainWindow::calculate_relativeWeight(int dataNumber){
    QVector<QVector<double> > out;
    QVector<double> tempDV;
    double tempD, tempD2;
    if (dataNumber<0)
        dataNumber=0;

    if (ui->_0_sb_model->value()<rawResDatas.size()&ui->_0_sb_model->value()>=0)
        if(rawResDatas.at(ui->_0_sb_model->value()).size()>dataNumber&dataNumber>=0)
            if(rawResHeaders.size()>ui->_0_sb_model->value()&ui->_0_sb_model->value()>=0){
    out.resize(this->rawResDatas.at(0).at(dataNumber).size());
    for (int i=0;i<endPoints.size();i++)
        endPoints[i].weightParser.generate_tree();

    if (ui->_2_cb_logScale->isChecked()){
        for (int j=0;j<this->rawResDatas.at(0).at(dataNumber).size();j++){
            out[j].resize(rawResDatas.size());
            tempDV.clear();
            tempD=0;
            bool base=false;
            tempD2=0;
            for (int i=0;i<this->rawResDatas.size();i++){
                if (!base&endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j))==endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j))){
                    tempD2=endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j));
                    base=true;
                }

                if (this->rawResDatas.at(i).at(dataNumber).size()>j){
//                    qDebug()<<this->rawResDatas.at(i).at(dataNumber).at(j);
                    tempDV<<endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j));
                    if (tempDV[i]==tempDV[i]){
                        tempDV[i]=exp(tempDV[i]-tempD2);
                    }else{
                        tempDV[i]=0;
                    }
                    tempD=tempD+(tempDV[i]);
                }else{
                    tempDV<<0;
                    tempD=tempD+(0);
                }
            }

            for (int i=0;i<this->rawResDatas.size();i++)
                out[j][i]=tempDV.at(i)/tempD;
        }
    }else{
        for (int j=0;j<this->rawResDatas.at(0).at(dataNumber).size();j++){
            out[j].resize(rawResDatas.size());
            tempDV.clear();
            tempD=0;
            for (int i=0;i<this->rawResDatas.size();i++){
                if (this->rawResDatas.at(i).at(dataNumber).size()>j){
                    tempDV<<endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j));
                    tempD=tempD+(tempDV[i]);
                }else{
                    tempDV<<0;
                    tempD=tempD+(0);
                }
            }

            for (int i=0;i<this->rawResDatas.size();i++)
                out[j][i]=tempDV.at(i)/tempD;
        }
    }
    }
    return out;
}

QVector<int> MainWindow::make_ModelSelection(int dataNumber){
    QVector<int> out;
    double tempD, reference;
    bool found;

    if (dataNumber<0)
        dataNumber=0;

    if(rawResDatas.size()>0)
        if(rawResDatas.at(0).size()>dataNumber){
            out.fill(-1,this->rawResDatas.at(0).at(dataNumber).size());
            for (int i=0;i<endPoints.size();i++)
                endPoints[i].weightParser.generate_tree();

            for (int j=0;j<this->rawResDatas.at(0).at(dataNumber).size();j++){
                tempD=0.0/0.0;
                found=false;
                for (int i=0;i<this->rawResDatas.size();i++){
                    if (this->rawResDatas.at(i).at(dataNumber).size()>j){
                        tempD=endPoints[i].evaluateWeight(this->rawResDatas.at(i).at(dataNumber).at(j));
                    }
                    if (tempD==tempD&!found){
                        reference=tempD;
                        out[j]=i;
                        found=true;
                    }else if(reference<tempD){
                        reference=tempD;
                        out[j]=i;
                    }

                }

            }
        }

    return out;
}

void MainWindow::on___mainToolBox_currentChanged(int index)
{
    if (index==3&ui->_2_tw_weightDefinition->rowCount()==0){

        refresh_2_tw_weightDefinition();
    }
}

void MainWindow::on__2_tw_weightDefinition_cellChanged(int row, int column)
{

    endPoints[row].weightParser.expression=ui->_2_tw_weightDefinition->item(row,column)->text();
    refresh_2_tw_relativeWeight();
}

void MainWindow::on__2_cb_logScale_stateChanged(int arg1)
{
    if (arg1==1){
        ui->_2_tw_weightDefinition->setHorizontalHeaderItem(0,new QTableWidgetItem("Weight Definition in log scale"));
    }else{
        ui->_2_tw_weightDefinition->setHorizontalHeaderItem(0,new QTableWidgetItem("Weight Definition"));
    }
    refresh_2_tw_relativeWeight();
}

void MainWindow::on__2_sb_dataNumber_valueChanged(int arg1)
{
    refresh_2_tw_relativeWeight();
}

void MainWindow::on__0_tw_RawResults_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn)
{
    refresh_0_cp_rawResHistoGram();
}


void MainWindow::refresh_0_cp_rawResHistoGram(){

    QPen tempPen;
    tempPen.setWidth(2);
    tempPen.setColor(uppsalaRed);
    int dataNumber=ui->_0_sb_data->value();
    if (dataNumber<0)
        dataNumber=0;

    qDebug()<<ui->_0_sb_model->value();

    if (this->rawResDatas.at(ui->_0_sb_model->value()).size()>0)
    if (ui->_0_tw_RawResults->currentColumn()>=0&ui->_0_tw_RawResults->currentColumn()<this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber).at(0).size()){
        QVector<double> tempDV, bin;
        tempDV.clear();
        bin.clear();

        int numBin=ceil(this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber).size()/5);

        for (int i=0;i<numBin;i++)
            bin<<ui->_0_cp_rawResHistoGram->xAxis->range().size()/numBin*i+ui->_0_cp_rawResHistoGram->xAxis->range().lower;

//        qDebug()<<bin;

        for (int i=0;i<this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber).size();i++)
            if (this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber).at(i).size()>ui->_0_tw_RawResults->currentColumn())
                tempDV<<this->rawResDatas.at(ui->_0_sb_model->value()).at(dataNumber).at(i).at(ui->_0_tw_RawResults->currentColumn());


        ui->_0_cp_rawResHistoGram->clearGraphs();

        histogram(tempDV,bin,ui->_0_cp_rawResHistoGram);

        ui->_0_cp_rawResHistoGram->graph(0)->setPen(tempPen);
        ui->_0_cp_rawResHistoGram->graph(0)->setBrush(this->uppsalaRedTrans);
        ui->_0_cp_rawResHistoGram->graph(0)->setChannelFillGraph(0);
        ui->_0_cp_rawResHistoGram->xAxis->setLabel(rawResHeaders.at(ui->_0_sb_model->value()).at(ui->_0_tw_RawResults->currentColumn()));
        ui->_0_cp_rawResHistoGram->yAxis->setLabel("Frequency");
        ui->_0_cp_rawResHistoGram->replot();
    }

}

//=====================================================================================
//=====================================================================================
//=====================================================================================
//DISTEFFECT PLOTS
//=====================================================================================
//=====================================================================================
//=====================================================================================


void MainWindow::mousePressDE()
{
  // if an axis is selected, only allow the direction of that axis to be dragged
  // if no axis is selected, both directions may be dragged

    ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(Qt::Horizontal);

//  if (ui->_0_cp_rawResHistoGram->xAxis->selectedParts().testFlag(QCPAxis::spAxis))
//      ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(Qt::Horizontal);
//    //ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(ui->_0_cp_rawResHistoGram->xAxis->orientation());
//  else if (ui->_0_cp_rawResHistoGram->yAxis->selectedParts().testFlag(QCPAxis::spAxis))
//        ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(Qt::Vertical);
//    //ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(ui->_0_cp_rawResHistoGram->yAxis->orientation());
//  else
//    ui->_0_cp_rawResHistoGram->axisRect()->setRangeDrag(Qt::Horizontal|Qt::Vertical);
  refresh_0_cp_rawResHistoGram();

}

void MainWindow::mouseWheelDE()
{
  // if an axis is selected, only allow the direction of that axis to be zoomed
  // if no axis is selected, both directions may be zoomed
    ui->_0_cp_rawResHistoGram->axisRect()->setRangeZoom(Qt::Horizontal|Qt::Vertical);


//  if (ui->_0_cp_rawResHistoGram->xAxis->selectedParts().testFlag(QCPAxis::spAxis))
//    ui->_0_cp_rawResHistoGram->axisRect()->setRangeZoom(ui->_0_cp_rawResHistoGram->xAxis->orientation());
//  else if (ui->_0_cp_rawResHistoGram->yAxis->selectedParts().testFlag(QCPAxis::spAxis))
//    ui->_0_cp_rawResHistoGram->axisRect()->setRangeZoom(ui->_0_cp_rawResHistoGram->yAxis->orientation());
//  else
//    ui->_0_cp_rawResHistoGram->axisRect()->setRangeZoom(Qt::Horizontal|Qt::Vertical);

  refresh_0_cp_rawResHistoGram();
}

void MainWindow::mouseWheel_4_cp_doseDist(){
    ui->_4_cp_doseChoiceDistribution->axisRect()->setRangeDrag(Qt::Horizontal);
    refresh_4_tw_doseChoice();
}

void MainWindow::mousePress_4_cp_doseDist(){
    ui->_4_cp_doseChoiceDistribution->axisRect()->setRangeZoom(Qt::Horizontal|Qt::Vertical);
    refresh_4_tw_doseChoice();
}

void MainWindow::on__1_le_possibleDoses_textChanged(const QString &arg1)
{
    QStringList tempQSL;
    bool b1,b2,b3;
    double lowerBound, interval, upperBound, tempD;

    QVector<double> tempDV;

    tempQSL=arg1.split(":",QString::SkipEmptyParts);
    if(tempQSL.size()==3){
        lowerBound=tempQSL[0].toDouble(&b1);
        interval=tempQSL[1].toDouble(&b2);
        upperBound=tempQSL[2].toDouble(&b3);

        if(b1&b2&b3&interval!=0)
            for(int i=0;i<ceil((upperBound-lowerBound)/interval)+1;i++)
                tempDV<<lowerBound+i*interval;
    }else{
        tempQSL=arg1.split(",",QString::SkipEmptyParts);
        tempQSL.removeDuplicates();

        for (int i=0; i<tempQSL.size();i++){
            tempD=tempQSL[i].toDouble(&b1);

            if (b1)
                tempDV<<tempD;
        }
    }

    if (tempDV.size()>0)
        qSort(tempDV.begin(),tempDV.end());
    independentVariables=tempDV;

    refresh_1_cp_typicalDRPlot();

    if (independentVariables.size()>0)
        ui->_3_sl_doseSlider->setMaximum(independentVariables.size()-1);

}

void MainWindow::refresh_1_cp_typicalDRPlot(){
    QVector<double> x,y;
    QPen tempPen;
    tempPen.setWidth(2);
    tempPen.setColor(uppsalaRed);

    if (independentVariables.size()>1&ui->_1_tw_endPoints->currentRow()>=0&ui->_1_tw_endPoints->currentRow()<endPoints.size()){
        ui->_1_cp_typicalDRPlot->clearGraphs();
        ui->_1_cp_typicalDRPlot->addGraph();
        endPoints[ui->_1_tw_endPoints->currentRow()].indepVariables=independentVariables;
        x=independentVariables;
        endPoints[ui->_1_tw_endPoints->currentRow()].epParser.generate_tree();
        y=endPoints[ui->_1_tw_endPoints->currentRow()].evaluateEndpoint(this->rawResDatas.at(ui->_1_tw_endPoints->currentRow()).at(0).at(0));
        ui->_1_cp_typicalDRPlot->graph(0)->setData(x,y);

        qSort(x.begin(),x.end());
        qSort(y.begin(),y.end());
        ui->_1_cp_typicalDRPlot->xAxis->setRange(x.first(),x.last());
        ui->_1_cp_typicalDRPlot->xAxis->setLabel(endPoints[ui->_1_tw_endPoints->currentRow()].epParser.variable_names.at(2));
        ui->_1_cp_typicalDRPlot->yAxis->setRange(y.first(),y.last());
        ui->_1_cp_typicalDRPlot->yAxis->setLabel("Endpoint");
        ui->_1_cp_typicalDRPlot->graph(0)->setPen(tempPen);

        ui->_1_cp_typicalDRPlot->replot();
    }
}

void MainWindow::on__3_sl_doseSlider_valueChanged(int value)
{
    ui->_3_lb_indpVar->setText("Dose (Independent Variable) = "+QString::number(this->independentVariables.at(value)));
}

void MainWindow::refresh_3_cp_PoS(){

    QPen tempPen;
    tempPen.setWidth(2);
    tempPen.setColor(uppsalaRed);

    int dataNumber=ui->_3_cb_dataNumber->value();

    QVector<double> x , y;

    if (this->independentVariables.size()>0){


    QVector<QVector<QVector<double> > > endpoints;
    QVector<int> selectedIndecies;
    QVector<QVector<double> > weights;
    y.fill(0,this->independentVariables.size());

    ui->_3_cp_PoS->xAxis->setRange(this->independentVariables.at(0),this->independentVariables.last());
    ui->_3_cp_PoS->xAxis->setLabel(endPoints[0].epParser.variable_names.at(2));
    ui->_3_cp_PoS->yAxis->setRange(0,100);
    ui->_3_cp_PoS->yAxis->setLabel("Probability");

    for(int i=0;i<endPoints.size();i++)
        endPoints[i].epParser.generate_tree();

    QTime dieTime;


    if (ui->_3_cb_aveORselect->currentIndex()==1){
        selectedIndecies=this->make_ModelSelection(dataNumber);
        endpoints.resize(1);

        QProgressDialog progress("Calculating probability of success.", "Abort Calculation", 0, this->rawResDatas.at(0).at(dataNumber).size(), this);
        progress.setWindowModality(Qt::WindowModal);

        for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
            dieTime= QTime::currentTime().addMSecs(10);

            ui->_3_cp_PoS->clearGraphs();
            ui->_3_cp_PoS->addGraph();

            if (selectedIndecies.at(i)>=0&selectedIndecies.at(i)<=this->rawResDatas.size())
                if (ui->_3_cb_indOFVone->currentIndex()==0){
                    endPoints[selectedIndecies.at(0)].indepVariables=independentVariables;
                    endpoints[0]<<endPoints[selectedIndecies.at(0)].evaluateEndpoint(this->rawResDatas.at(selectedIndecies.at(0)).at(dataNumber).at(i));
                }else{
                    endPoints[selectedIndecies.at(i)].indepVariables=independentVariables;
                    endpoints[0]<<endPoints[selectedIndecies.at(i)].evaluateEndpoint(this->rawResDatas.at(selectedIndecies.at(i)).at(dataNumber).at(i));
                }

            ui->_3_cp_PoS->graph(0)->addData(this->independentVariables,calculate_PoS_aux_modelSelect(endpoints));

            ui->_3_cp_PoS->graph(0)->setPen(tempPen);

            ui->_3_cp_PoS->replot();
            progress.setValue(i);
            progress.setLabelText("Calculating probability of success using bootstrap samples 0-"+QString::number(i)+".");
            if (progress.wasCanceled())
                break;

            while( QTime::currentTime() < dieTime )
            QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
        }
//        y=calculate_PoS_aux_modelSelect(endpoints);

    }else{
        QProgressDialog progress("Calculating probability of success.", "Abort Calculation", 0, this->rawResDatas.at(0).at(dataNumber).size(), this);
        progress.setWindowModality(Qt::WindowModal);

        weights=calculate_relativeWeight(dataNumber);
        selectedIndecies=this->make_ModelSelection(dataNumber);
        endpoints.resize(endPoints.size());
        for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
            dieTime= QTime::currentTime().addMSecs(10);

            ui->_3_cp_PoS->clearGraphs();
            ui->_3_cp_PoS->addGraph();

            if (selectedIndecies.at(i)>=0&selectedIndecies.at(i)<=this->rawResDatas.size())
                for (int j=0;j<endPoints.size();j++){
                    endPoints[j].indepVariables=independentVariables;
                    if ((ui->_3_cb_indOFVone->currentIndex()==0& weights.at(0).at(j)>0)|(ui->_3_cb_indOFVone->currentIndex()==1& weights.at(i).at(j)>0)){
                        endpoints[j]<<endPoints[j].evaluateEndpoint(this->rawResDatas.at(j).at(dataNumber).at(i));
                    }else{
                        endpoints[j]<<x;
                    }
                }

            ui->_3_cp_PoS->graph(0)->addData(this->independentVariables,calculate_PoS_aux_modelAve(endpoints, weights));

            ui->_3_cp_PoS->graph(0)->setPen(tempPen);

            ui->_3_cp_PoS->replot();
            progress.setValue(i);
            progress.setLabelText("Calculating probability of success using bootstrap samples 0-"+QString::number(i)+".");
            if (progress.wasCanceled())
                break;

            while( QTime::currentTime() < dieTime )
            QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
        }
//        y=calculate_PoS_aux_modelAve(endpoints, weights);

    }
    }


}

QVector<double> MainWindow::calculate_PoS(int dataNumber){
    QVector<double> x , y;
    if (dataNumber<0)
        dataNumber=0;

    if (this->independentVariables.size()>0){


        QVector<QVector<QVector<double> > > endpoints;
        QVector<int> selectedIndecies;
        QVector<QVector<double> > weights;
        y.fill(0,this->independentVariables.size());

        for(int i=0;i<endPoints.size();i++){
            endPoints[i].epParser.generate_tree();
            endPoints[i].indepVariables=independentVariables;
        }

        if(ui->_4_cb_useCash->isChecked()){

            bool createCash=false;

            if (this->cash_endpoints.size()>dataNumber){
                if(this->cash_endpoints.at(dataNumber).size()!=endPoints.size()){
                    createCash=true;
                    this->cash_endpoints[dataNumber].clear();
                }
            }else{
                createCash=true;
                this->cash_endpoints.resize(dataNumber+1);
            }
            if(createCash){
                this->cash_endpoints[dataNumber].clear();
                endpoints.clear();
                endpoints.resize(endPoints.size());

                for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
                    for (int j=0;j<endPoints.size();j++){

                        endpoints[j]<<endPoints[j].evaluateEndpoint(this->rawResDatas.at(j).at(dataNumber).at(i));

                    }
                }
                this->cash_endpoints[dataNumber]=endpoints;
            }

            endpoints.clear();

            if (ui->_3_cb_aveORselect->currentIndex()==1){
                selectedIndecies=this->make_ModelSelection(dataNumber);
                endpoints.resize(1);
                for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
                    if (selectedIndecies.at(i)>=0&selectedIndecies.at(i)<=this->rawResDatas.size())
                        if (ui->_3_cb_indOFVone->currentIndex()==0){
                            endpoints[0]<<cash_endpoints[dataNumber][selectedIndecies.at(0)][i];
                        }else{
                            //qDebug()<<cash_endpoints;
                            //qDebug()<<selectedIndecies.at(i);
                            endpoints[0]<<cash_endpoints[dataNumber][selectedIndecies.at(i)][i];;
                        }
                }

                y=calculate_PoS_aux_modelSelect(endpoints);

            }else{
                weights=calculate_relativeWeight(dataNumber);
                selectedIndecies=this->make_ModelSelection(dataNumber);
                endpoints=this->cash_endpoints[dataNumber];
                y=calculate_PoS_aux_modelAve(endpoints,weights);

            }
        }else{

            if (ui->_3_cb_aveORselect->currentIndex()==1){
                selectedIndecies=this->make_ModelSelection(dataNumber);
                endpoints.resize(1);


                for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
                    if (selectedIndecies.at(i)>=0&selectedIndecies.at(i)<=this->rawResDatas.size())
                        if (ui->_3_cb_indOFVone->currentIndex()==0){
                            endPoints[selectedIndecies.at(0)].indepVariables=independentVariables;
                            endpoints[0]<<endPoints[selectedIndecies.at(0)].evaluateEndpoint(this->rawResDatas.at(selectedIndecies.at(0)).at(dataNumber).at(i));
                        }else{
                            endPoints[selectedIndecies.at(i)].indepVariables=independentVariables;
                            endpoints[0]<<endPoints[selectedIndecies.at(i)].evaluateEndpoint(this->rawResDatas.at(selectedIndecies.at(i)).at(dataNumber).at(i));
                        }
                }
                y=calculate_PoS_aux_modelSelect(endpoints);

                }else{
                    weights=calculate_relativeWeight(dataNumber);
                    selectedIndecies=this->make_ModelSelection(dataNumber);
                    endpoints.resize(endPoints.size());
                    if (ui->_4_cb_useCash->isChecked()&endPoints.size()==cash_endpoints[dataNumber].size()){
                        endpoints=this->cash_endpoints[dataNumber];
                    }else{
                        for (int i=0; i<this->rawResDatas.at(0).at(dataNumber).size();i++){
                            if (selectedIndecies.at(i)>=0&selectedIndecies.at(i)<=this->rawResDatas.size())
                                for (int j=0;j<endPoints.size();j++){
                                    endPoints[j].indepVariables=independentVariables;

                                    if ((ui->_3_cb_indOFVone->currentIndex()==0& weights.at(0).at(j)>0)|(ui->_3_cb_indOFVone->currentIndex()==1& weights.at(i).at(j)>0)){
                                        endpoints[j]<<endPoints[j].evaluateEndpoint(this->rawResDatas.at(j).at(dataNumber).at(i));
                                    }else{
                                        endpoints[j]<<x;
                                    }

                                }
                        }
                        if(ui->_4_cb_useCash->isChecked()){
                            this->cash_endpoints[dataNumber]=endpoints;
                        }

                    }
                    y=calculate_PoS_aux_modelAve(endpoints,weights);

                }
        }
    }

    return y;
}


QVector<double> MainWindow::calculate_PoS_aux_modelSelect(QVector<QVector<QVector<double> > > endpoints){

    QVector<double> y;
    y.fill(0,this->independentVariables.size());

    for(int i=0; i<endpoints[0].size();i++){
        for (int j=0; j<endpoints[0].at(i).size(); j++){
            if (endpoints[0].at(i).at(j)>this->targetLowerbound&endpoints[0].at(i).at(j)<=this->targetUpperbound)
                y[j]=y.at(j)+1.0/(double)endpoints[0].size()*100;
        }
    }
    return y;
}

QVector<double> MainWindow::calculate_PoS_aux_modelAve(QVector<QVector<QVector<double> > > endpoints, QVector<QVector<double> > weights){

    QVector<double> y;
    y.fill(0,this->independentVariables.size());

    for(int i=0; i<endpoints[0].size();i++){
        for (int k=0;k<endPoints.size();k++){
            for (int j=0; j<endpoints[k].at(i).size(); j++){
                //qDebug()<<endpoints[k].at(i).at(j);

                    if (ui->_3_cb_indOFVone->currentIndex()==0){
                        if(weights.at(0).at(k)>0)
                            if (endpoints[k].at(i).at(j)>this->targetLowerbound&endpoints[k].at(i).at(j)<=this->targetUpperbound)
                                y[j]=y.at(j)+weights.at(0).at(k)/(double)endpoints[k].size()*100;
                    }else{
                        if (weights.at(i).at(k)>0)
                            if (endpoints[k].at(i).at(j)>this->targetLowerbound&endpoints[k].at(i).at(j)<=this->targetUpperbound)
                                y[j]=y.at(j)+weights.at(i).at(k)/(double)endpoints[k].size()*100;
                    }
            }
        }
    }

    return y;
}

QVector<double> MainWindow::find_dose(QVector<double> PoS,QVector<double> probabilities){
    QVector<double> out;
    out.resize(probabilities.size());
    int index=PoS.size()-1;
    for (int i=0;i<probabilities.size(); i++){
        int tempInt=find_dose(PoS,probabilities.at(i));
        if(tempInt<independentVariables.size()&tempInt>=0){
            out[i]=this->independentVariables.at(tempInt);
        }else{
            out[i]=0.0/0.0;
        }
    }
    return out;

}

QVector<double> MainWindow::find_dose(QVector<double> PoS){
    QVector<double> probabilities=this->dsProbabilities;
    QVector<double> out;
    out.resize(probabilities.size());
    int index=PoS.size()-1;
    for (int i=0;i<probabilities.size(); i++){
        int tempInt=find_dose(PoS,probabilities.at(i));
        if(tempInt<independentVariables.size()&tempInt>=0){
            out[i]=this->independentVariables.at(tempInt);
        }else{
            out[i]=0.0/0.0;
        }
    }
    return out;

}

int MainWindow::find_dose(QVector<double> PoS,double probability){
    int index=PoS.size()-1;

    if (index>=0)
        while (probability<PoS.at(index)){
            index--;
            if (index<0){
                break;
            }
        }
    return index+1;

}

void MainWindow::on__3_cb_dataNumber_valueChanged(int arg1)
{
   // refresh_3_cp_PoS();
}

void MainWindow::on__3_cb_aveORselect_currentIndexChanged(int index)
{
   // refresh_3_cp_PoS();
}

void MainWindow::on__3_cb_indOFVone_currentIndexChanged(int index)
{
   // refresh_3_cp_PoS();
}

void MainWindow::on__4_pb_startCalculation_clicked()
{
    if (rawResDatas.size()>0){
    QProgressDialog progress("Calculating...", "Abort Calculation", 0, rawResDatas.at(0).size(), this);
         progress.setWindowModality(Qt::WindowModal);

    QString arg1=ui->_4_le_probabilities->text();
    QStringList tempQSL;
    bool b1,b2,b3;
    double lowerBound, interval, upperBound, tempD;

    QVector<double> probabilities, tempDV, tempDV2;
    QVector<QVector<double> > tempDMat;
    this->selectedDoses.clear();
    this->dsProbabilitiesSL.clear();

    tempQSL=arg1.split(":",QString::SkipEmptyParts);
    if(tempQSL.size()==3){
        lowerBound=tempQSL[0].toDouble(&b1);
        interval=tempQSL[1].toDouble(&b2);
        upperBound=tempQSL[2].toDouble(&b3);

        if(b1&b2&b3)
            for(int i=0;i<ceil((upperBound-lowerBound)/interval)+1;i++)
                tempDV<<lowerBound+i*interval;
    }else{
        tempQSL=arg1.split(",",QString::SkipEmptyParts);
        tempQSL.removeDuplicates();

        for (int i=0; i<tempQSL.size();i++){
            tempD=tempQSL[i].toDouble(&b1);

            if (b1)
                tempDV<<tempD;
        }
    }

    if (tempDV.size()>0)
        qSort(tempDV.begin(),tempDV.end());

    this->dsProbabilities=tempDV;

  //  tempQSL.clear();
    for (int i=0;i<probabilities.size();i++)
        this->dsProbabilitiesSL<<QString::number(probabilities[i]);

    QVector<int> tempQVI;
    for (int i=0;i<rawResDatas.at(0).size();i++)
        tempQVI<<i;

//    QFuture<QVector<double> > calculated_PoS=QtConcurrent::map(tempQVI.begin(),tempQVI.end(),calculate_PoS);
//    progress.setValue(calculated_PoS.progressValue());

    if (ui->_4_cb_useCash->isChecked())
        cash_endpoints.resize(rawResDatas.at(0).size());
    for (int i=0;i<rawResDatas.at(0).size();i++){
        this->selectedDoses<<find_dose(calculate_PoS(i));
        csvDisplay(ui->_4_tw_doseChoice,this->selectedDoses,this->dsProbabilitiesSL);
        progress.setValue(i+1);
        if (progress.wasCanceled())
                     break;
    }

//    csvwrite(ui->__lb_directory->text()+"/doseChoiceWithProb.csv",this->selectedDoses,this->dsProbabilitiesSL);
    }

}

void MainWindow::on__4_pb_compTruth_clicked()
{QVector<QVector<double> > count, underOverEst;
    if(simRawResDatas.size()==0){
        theoreticalMED.fill(0.0/0.0,this->simRawResDatas.size());

        QString tempFileName= QFileDialog::getOpenFileName(this,"Import file with true model and true MED.",ui->__lb_directory->text(),"*.csv");
        QVector<QVector<double> > tempDM=csvReadToDouble(tempFileName);
        if(this->rawResDatas.size()>0)
            if (tempDM.size()==this->rawResDatas.at(0).size())
                if(tempDM.at(0).size()>=2){
                    csvDisplay(ui->_5_tw_trueModelTMED, tempDM);
                    theoreticalMED.clear();
                    theoreticalMED.fill(0.0/0.0,tempDM.size());

                    for (int i=0;i<tempDM.size()&i<theoreticalMED.size();i++){
                        int j=independentVariables.size()-1;
                        while(tempDM.at(i).at(1)<=independentVariables.at(j)){
                            j--;
                            if (j<0)
                                break;
                        }
                        if(j+1<independentVariables.size())
                            theoreticalMED[i]=this->independentVariables.at(j+1);
                    }
                }

    //    QVector<QVector<double> > estimates=csvWithHeaderReadToDouble(ui->__lb_directory->text()+"/doseChoiceWithProb.csv");


        if(selectedDoses.size()>0){
            count.resize(selectedDoses.at(0).size());
            underOverEst.resize(selectedDoses.at(0).size());

            for (int i=0; i<count.size();i++){
                count[i].fill(0,3);
                underOverEst[i].fill(0,1);
                int count1, count2;
                count1=0;
                count2=0;

                for (int j=0;j<selectedDoses.size()&j<theoreticalMED.size();j++){
                    if (selectedDoses.at(j).at(i)<(theoreticalMED.at(j))){
                        count[i][2]++;
                        if (selectedDoses.at(j).at(i)==selectedDoses.at(j).at(i)){
                            underOverEst[i][0]=underOverEst[i][0]+(theoreticalMED.at(j)-selectedDoses.at(j).at(i));
                            count1++;
                        }
                    }else if(selectedDoses.at(j).at(i)==(theoreticalMED.at(j))){
                        count[i][1]++;
                    }else{
                        count[i][0]++;
                        if (selectedDoses.at(j).at(i)==selectedDoses.at(j).at(i)){
                            underOverEst[i][0]=underOverEst[i][0]+(selectedDoses.at(j).at(i)-theoreticalMED.at(j));
                            count2++;
                        }
                    }
                }
                underOverEst[i][0]=underOverEst[i][0]/(count1+count2);
            }

            for(int i=0;i<theoreticalMED.size()&i<ui->_5_tw_trueModelTMED->rowCount();i++)
                ui->_5_tw_trueModelTMED->setItem(i,1,new QTableWidgetItem(QString::number(theoreticalMED.at(i))));

//            csvwrite(ui->__lb_directory->text()+"/count.csv",count);
        }
    }else{

        simulationResults.clear();
        theoreticalMED.clear();

        for(int i=0;i<endPoints.size();i++){
            endPoints[i].indepVariables=this->independentVariables;
            endPoints[i].epParser.generate_tree();
        }
        trueModels.clear();
        bool isInt;
        bool issueWarning=false;
        for(int i=0;i<ui->_5_tw_trueModelTMED->rowCount();i++){
            if(ui->_5_tw_trueModelTMED->item(i,0)!=0){
                trueModels<<ui->_5_tw_trueModelTMED->item(i,0)->text().toInt(&isInt);
            }else{
                issueWarning=true;
                break;
            }
            if(!isInt|trueModels.last()<0|trueModels.last()>=endPoints.size()){
                issueWarning=true;
                break;
            }
        }



        if(issueWarning&independentVariables.size()>0){
            QMessageBox warning;
            warning.setText("Could not read in the true model index correctly.");

        }else{
            theoreticalMED.fill(0.0/0.0,this->simRawResDatas.size());
            for(int i=0;i<this->simRawResDatas.size();i++){
                simulationResults<<endPoints[this->trueModels.at(i)].evaluateEndpoint(this->simRawResDatas.at(i).at(0));
                int j=independentVariables.size()-1;
                while(simulationResults.last().at(j)>this->targetLowerbound&simulationResults.last().at(j)<=this->targetUpperbound){
                    j--;
                    if (j<0)
                        break;
                }
                if(j+1<independentVariables.size())
                    theoreticalMED[i]=this->independentVariables.at(j+1);
            }

            for(int i=0;i<theoreticalMED.size()&i<ui->_5_tw_trueModelTMED->rowCount();i++)
                ui->_5_tw_trueModelTMED->setItem(i,1,new QTableWidgetItem(QString::number(theoreticalMED.at(i))));

       //     QVector<QVector<double> > estimates=csvWithHeaderReadToDouble(ui->__lb_directory->text()+"/doseChoiceWithProb.csv");



            if(selectedDoses.size()>0){
            count.resize(selectedDoses.at(0).size());
            underOverEst.resize(selectedDoses.at(0).size());


            for (int i=0; i<count.size();i++){
                count[i].fill(0,3);
                underOverEst[i].fill(0,1);

                int count1, count2;
                count1=0;
                count2=0;

                for (int j=0;j<selectedDoses.size()&j<theoreticalMED.size();j++){
                    if (selectedDoses.at(j).at(i)<(theoreticalMED.at(j))){
                        count[i][2]++;
                        if (selectedDoses.at(j).at(i)==selectedDoses.at(j).at(i)){
                            underOverEst[i][0]=underOverEst[i][0]+(theoreticalMED.at(j)-selectedDoses.at(j).at(i));
                            count1++;
                        }
                    }else if(selectedDoses.at(j).at(i)==(theoreticalMED.at(j))){
                        count[i][1]++;
                    }else{
                        count[i][0]++;
                        if (selectedDoses.at(j).at(i)==selectedDoses.at(j).at(i)){
                            underOverEst[i][0]=underOverEst[i][0]+(selectedDoses.at(j).at(i)-theoreticalMED.at(j));
                            count2++;
                        }
                    }
                }
                underOverEst[i][0]=underOverEst[i][0]/(count1+count2);

            }

            //csvwrite(ui->__lb_directory->text()+"/count.csv",count);
            }
        }
    }

    QVector<double> y1, y2, y3, y4;
    QVector<QVector<double> > simSummary;

    simSummary.resize(6);
    simSummary[0].fill(0,2);
    simSummary[1].fill(0,2);
    simSummary[2].fill(0,2);
    simSummary[3].fill(0,2);
    simSummary[4].fill(0,2);
    simSummary[5].fill(0,2);

    int tempCounter=0;
    int tempCounter2=0;
    double temp1, temp2, temp3, temp4, temp5, temp6;

    temp1=0;
    temp2=0;
    temp3=0;
    temp4=0;
    temp5=0;
    temp6=0;

    for (int i=0;i<count.size()&i<dsProbabilities.size();i++){

        y1<<count.at(i).at(0);
        y2<<count.at(i).at(0)+count.at(i).at(1);
        y3<<count.at(i).at(0)+count.at(i).at(1)+count.at(i).at(2);
        y4<<y3.last()*dsProbabilities.at(i)/100;

        if (dsProbabilities.at(i)>0&dsProbabilities.at(i)<100){
            tempCounter2++;

            if(y4.last()>y2.last()){
                simSummary[2][0]=simSummary[2][0]+(y4.last()-y2.last());
                if (temp3<(y4.last()-y2.last()))
                    temp3=(y4.last()-y2.last());
            }

            if(y4.last()<y1.last()){
                simSummary[0][0]=simSummary[0][0]+(y1.last()-y4.last());
                if (temp1<(y1.last()-y4.last()))
                    temp1=(y1.last()-y4.last());

            }

            simSummary[1][0]=simSummary[1][0]+count.at(i).at(1);
            if (temp2<count.at(i).at(1))
                temp2=count.at(i).at(1);



            if (dsProbabilities.at(i)>=50){
                tempCounter++;
                if(y4.last()>y2.last()){
                    simSummary[2][1]=simSummary[2][1]+(y4.last()-y2.last());
                    if (temp6<(y4.last()-y2.last()))
                        temp6=(y4.last()-y2.last());
                }

                if(y4.last()<y1.last()){
                    simSummary[0][1]=simSummary[0][1]+(y1.last()-y4.last());
                    if (temp4<(y1.last()-y4.last()))
                        temp4=(y1.last()-y4.last());
                }

                simSummary[1][1]=simSummary[1][1]+count.at(i).at(1);
                if (temp5<count.at(i).at(1))
                    temp5=count.at(i).at(1);
            }
        }
    }

    simSummary[0][0]=simSummary[0][0]/tempCounter2;
    simSummary[1][0]=simSummary[1][0]/tempCounter2;
    simSummary[2][0]=simSummary[2][0]/tempCounter2;

    simSummary[0][1]=simSummary[0][1]/tempCounter;
    simSummary[1][1]=simSummary[1][1]/tempCounter;
    simSummary[2][1]=simSummary[2][1]/tempCounter;

    simSummary[3][0]=temp1;
    simSummary[4][0]=temp2;
    simSummary[5][0]=temp3;
    simSummary[3][1]=temp4;
    simSummary[4][1]=temp5;
    simSummary[5][1]=temp6;

    csvDisplay(ui->_5_tw_Summary,simSummary);

    if (y1.size()>0){
        QBrush myBrush;
        QPen myPen;
        myBrush.setColor(Qt::yellow);
        myPen.setColor(Qt::white);

        QColor tempColour=Qt::yellow;
        tempColour.setAlpha(150);
        ui->_5_cp_truthVisualization->clearGraphs();
        ui->_5_cp_truthVisualization->addGraph();
        ui->_5_cp_truthVisualization->addGraph();
        ui->_5_cp_truthVisualization->graph(1)->setData(dsProbabilities,y1);
        ui->_5_cp_truthVisualization->graph(1)->setBrush(tempColour);
        ui->_5_cp_truthVisualization->graph(1)->setPen(myPen);
        ui->_5_cp_truthVisualization->graph(1)->setChannelFillGraph(0);
        ui->_5_cp_truthVisualization->graph(1)->channelFillGraph();
        ui->_5_cp_truthVisualization->addGraph();
        ui->_5_cp_truthVisualization->graph(2)->setData(dsProbabilities,y2);
        tempColour=Qt::green;
        tempColour.setAlpha(150);
        ui->_5_cp_truthVisualization->graph(2)->setBrush(tempColour);
        ui->_5_cp_truthVisualization->graph(2)->setPen(myPen);
        ui->_5_cp_truthVisualization->graph(2)->setChannelFillGraph(ui->_5_cp_truthVisualization->graph(1));
        ui->_5_cp_truthVisualization->graph(2)->channelFillGraph();
        ui->_5_cp_truthVisualization->addGraph();
        ui->_5_cp_truthVisualization->graph(3)->setData(dsProbabilities,y3);
        tempColour=Qt::red;
        tempColour.setAlpha(150);
        ui->_5_cp_truthVisualization->graph(3)->setBrush(tempColour);
        ui->_5_cp_truthVisualization->graph(3)->setPen(myPen);
        ui->_5_cp_truthVisualization->graph(3)->setChannelFillGraph(ui->_5_cp_truthVisualization->graph(2));
        ui->_5_cp_truthVisualization->graph(3)->channelFillGraph();
        ui->_5_cp_truthVisualization->graph(0)->setData(dsProbabilities,y4);
        myPen.setWidth(2);
        myPen.setColor(Qt::black);

        ui->_5_cp_truthVisualization->graph(0)->setPen(myPen);
        ui->_5_cp_truthVisualization->yAxis->setRange(0,y3.first());
        ui->_5_cp_truthVisualization->xAxis->setRange(dsProbabilities.first(),dsProbabilities.last());
        ui->_5_cp_truthVisualization->replot();
    }

    csvDisplay(ui->_5_tw_aveUnderEstOverEst,underOverEst);

}

void MainWindow::on___pb_importFbF_clicked()
{
    QStringList raw_res_files;
    raw_res_files=QFileDialog::getOpenFileNames(this, ("Select PsN bootstrap raw_results files."));
    raw_res_files.sort();
    if (raw_res_files.size()>0){
        QStringList tempQSL=raw_res_files.at(0).split("/");
        tempQSL.removeLast();

        ui->__lb_directory->setText(tempQSL.join("/"));
    }
    int currentNumModels=0;//this->rawResHeaders.size();

    if (this->rawResDatas.size()>0)
        if(this->rawResDatas.at(0).size()>1){
            this->rawResDatas.clear();
            this->rawResHeaders.clear();
        }

    this->rawResDatas.resize(currentNumModels+raw_res_files.size());
    for (int i=currentNumModels; i<currentNumModels+raw_res_files.size();i++){
        this->rawResHeaders<<csvWithHeaderReadHeader(raw_res_files.at(i-currentNumModels));
        this->rawResDatas[i]<<csvWithHeaderReadToDouble(raw_res_files.at(i-currentNumModels));
    }


    if (this->rawResDatas.size()==0){
        QMessageBox newMessage;
        newMessage.setText("raw results file cannot be found");
        newMessage.exec();
        ui->__mainToolBox->setCurrentIndex(0);
    }else{
        ui->__mainToolBox->setVisible(true);
        ui->_0_sb_model->setMaximum(this->rawResDatas.size()-1);
        ui->_1_tw_endPoints->setRowCount(this->rawResDatas.size());
        ui->_0_sb_data->setMaximum(0);
        ui->_0_sb_data->setVisible(false);
        ui->_2_sb_dataNumber->setMaximum(0);
        ui->_2_sb_dataNumber->setVisible(false);
        ui->_3_cb_dataNumber->setMaximum(0);
        ui->_3_cb_dataNumber->setVisible(false);
        ui->_0_lb_dataNumber->setVisible(false);
        ui->_2_lb_dataNumber->setVisible(false);
        ui->_3_lb_dataNumber->setVisible(false);

        this->endPoints.resize(this->rawResDatas.size());

        QStringList tempQSL;
        for (int i=currentNumModels;i<currentNumModels+raw_res_files.size();i++){
            this->endPoints[i].name=raw_res_files.at(i-currentNumModels).split("/").last().split("raw_results_").last().split(".csv").first();
            tempQSL<<this->endPoints.at(i).name;
            endPoints[i].OFV_index=this->OFV_index;
            endPoints[i].THETA_index=this->THETA_index;
            endPoints[i].ETA_index=this->ETA_index;

        }
        ui->_1_tw_endPoints->setVerticalHeaderLabels(tempQSL);
        refresh_0_tw_RawResults();
        ui->_2_tw_weightDefinition->setRowCount(this->rawResDatas.size());
        ui->_2_tw_relativeWeight->setHorizontalHeaderLabels(tempQSL);
        ui->_2_tw_weightDefinition->setVerticalHeaderLabels(tempQSL);
    }

}

void MainWindow::on__1_tw_endPoints_cellChanged(int row, int column)
{
    this->endPoints[row].epParser.expression=ui->_1_tw_endPoints->item(row, 0)->text();
    this->endPoints[row].epParser.extract_variables();
    refresh_1_cp_typicalDRPlot();
    refresh__1_tw_etaDefinition();

}

void MainWindow::refresh__1_tw_etaDefinition(){

    int row=ui->_1_tw_endPoints->currentRow();

    if(row>=0&row<endPoints.size()){
        this->endPoints[row].checkETA();
        if(this->endPoints[row].randomEffect){
            ui->_1_tw_etaDefinition->setVisible(true);
            ui->_1_lb_etaText->setVisible(true);
            ui->_1_cb_numEtas->setVisible(true);
            ui->_1_cb_iivEndPoint->setVisible(true);
            ui->_1_sb_numSimulations->setVisible(true);

            ui->_1_cb_numEtas->setMinimum(this->endPoints[row].currentETAmax);
            for (int i=0;i<endPoints[row].omegaIndecies.size();i++){
                for (int j=0;j<i+1;j++){
                    if(endPoints[row].omegaIndecies.at(i).at(j)>=0){
                       ui->_1_tw_etaDefinition->setItem(i,j,new QTableWidgetItem("COL"+QString::number(endPoints[row].omegaIndecies.at(i).at(j)+1)));
                       ui->_1_tw_etaDefinition->setItem(j,i,new QTableWidgetItem("COL"+QString::number(endPoints[row].omegaIndecies.at(i).at(j)+1)));

                    }else{
                       ui->_1_tw_etaDefinition->setItem(i,j,new QTableWidgetItem(QString::number(this->endPoints[row].omegaMatrix.at(i).at(j))));
                       ui->_1_tw_etaDefinition->setItem(j,i,new QTableWidgetItem(QString::number(this->endPoints[row].omegaMatrix.at(i).at(j))));
                    }
                }
            }
        }else{
            ui->_1_tw_etaDefinition->setVisible(false);
            ui->_1_lb_etaText->setVisible(false);
            ui->_1_cb_numEtas->setVisible(false);
            ui->_1_cb_iivEndPoint->setVisible(false);
            ui->_1_sb_numSimulations->setVisible(false);
        }
    }
}

void MainWindow::on__m1_le_colNumOFV_textEdited(const QString &arg1)
{
    bool isInt=false;
    arg1.toInt(&isInt);
    if(isInt){
        this->OFV_index=arg1.toInt(&isInt);
        for (int i=0;i<this->endPoints.size();i++)
            endPoints[i].OFV_index=this->OFV_index;
    }else{
        ui->_m1_le_colNumOFV->setText(QString::number(this->OFV_index));
    }
}

void MainWindow::on__m1_le_colNumTHETA1_textEdited(const QString &arg1)
{
    bool isInt=false;
    arg1.toInt(&isInt);
    if(isInt){
        this->THETA_index=arg1.toInt(&isInt);
        for (int i=0;i<this->endPoints.size();i++)
            endPoints[i].THETA_index=this->THETA_index;
    }else{
        ui->_m1_le_colNumTHETA1->setText(QString::number(this->THETA_index));
    }
}

void MainWindow::on__1_tw_endPoints_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn)
{
    ui->_1_tw_rawResLabels->setRowCount(1);
    ui->_1_tw_rawResLabels->setColumnCount(this->rawResHeaders.at(currentRow).size());
    for(int i=0;i<this->rawResHeaders.at(currentRow).size();i++)
        ui->_1_tw_rawResLabels->setItem(0,i,new QTableWidgetItem(this->rawResHeaders.at(currentRow).at(i)));
    refresh_1_cp_typicalDRPlot();
    refresh__1_tw_etaDefinition();
}

void MainWindow::on__1_tw_endPoints_itemChanged(QTableWidgetItem *item)
{
    this->endPoints[item->row()].epParser.expression=item->text();
    this->endPoints[item->row()].epParser.extract_variables();
    refresh_1_cp_typicalDRPlot();
    refresh__1_tw_etaDefinition();
}

void MainWindow::on__3_pb_calculatePoS_clicked()
{
    refresh_3_cp_PoS();
}

void MainWindow::on__3_le_lowerBound_editingFinished()
{
    bool ok;
    double tempD;

    QString arg1=ui->_3_le_lowerBound->text();

    tempD=arg1.toDouble(&ok);

    if(ok){
        this->targetLowerbound=tempD;
    }else{
        this->targetLowerbound=-1/0.1;
        ui->_3_le_lowerBound->setText("-inf");
    }
    //refresh_3_cp_PoS();
}

void MainWindow::on__3_le_upperBound_editingFinished()
{
    bool ok;
    double tempD;

    QString arg1=ui->_3_le_upperBound->text();

    tempD=arg1.toDouble(&ok);

    if(ok){
        this->targetUpperbound=tempD;
    }else{
        this->targetUpperbound=1/0.1;
        ui->_3_le_upperBound->setText("inf");
    }
    //refresh_3_cp_PoS();
}

void MainWindow::on__1_tw_etaDefinition_cellChanged(int row, int column)
{
    int modelIndex=ui->_1_tw_endPoints->currentRow();
    bool isDouble, isInt;
    int tempI;

    double tempD=ui->_1_tw_etaDefinition->item(row,column)->text().toDouble(&isDouble);

    endPoints[modelIndex].omegaMatrix[row][column]=tempD;
    endPoints[modelIndex].omegaMatrix[column][row]=tempD;
    if(!isDouble){

        tempI=ui->_1_tw_etaDefinition->item(row,column)->text().split("COL").last().toInt(&isInt)-1;
        if (isInt){
            endPoints[modelIndex].omegaIndecies[row][column]=tempI;
            endPoints[modelIndex].omegaIndecies[column][row]=tempI;
        }
        tempI=ui->_1_tw_etaDefinition->item(row,column)->text().split("Col").last().toInt(&isInt);
        if (isInt){
            endPoints[modelIndex].omegaIndecies[row][column]=tempI;
            endPoints[modelIndex].omegaIndecies[column][row]=tempI;
        }
        tempI=ui->_1_tw_etaDefinition->item(row,column)->text().split("COLUMN").last().toInt(&isInt);
        if (isInt){
            endPoints[modelIndex].omegaIndecies[row][column]=tempI;
            endPoints[modelIndex].omegaIndecies[column][row]=tempI;
        }
        tempI=ui->_1_tw_etaDefinition->item(row,column)->text().split("Column").last().toInt(&isInt);
        if (isInt){
            endPoints[modelIndex].omegaIndecies[row][column]=tempI;
            endPoints[modelIndex].omegaIndecies[column][row]=tempI;
        }
        tempI=ui->_1_tw_etaDefinition->item(row,column)->text().split("column").last().toInt(&isInt);
        if (isInt){
            endPoints[modelIndex].omegaIndecies[row][column]=tempI;
            endPoints[modelIndex].omegaIndecies[column][row]=tempI;
        }
    }else{

        endPoints[modelIndex].omegaIndecies[row][column]=-1;
        endPoints[modelIndex].omegaIndecies[column][row]=-1;

    }


    refresh_1_cp_typicalDRPlot();
}

void MainWindow::on__1_cb_numEtas_valueChanged(int arg1)
{   int row=ui->_1_tw_endPoints->currentRow();
    if (row>=0&row<endPoints.size())
    if (arg1<endPoints.at(row).omegaMatrix.size()){

        int oldMatrixSize=endPoints[row].omegaMatrix.size();
        endPoints[row].omegaMatrix.remove(arg1,oldMatrixSize-arg1);
        endPoints[row].omegaIndecies.remove(arg1,oldMatrixSize-arg1);

        for (int i=0;i<endPoints.at(row).omegaMatrix.size();i++){
            endPoints[row].omegaMatrix[i].remove(arg1,oldMatrixSize-arg1);
            endPoints[row].omegaIndecies[i].remove(arg1,oldMatrixSize-arg1);
        }


    }else if(arg1>endPoints.at(row).omegaMatrix.size()){

        int oldMatrixSize=endPoints[row].omegaMatrix.size();

        for (int j=0;j<this->endPoints.at(row).omegaMatrix.size();j++){
            for (int k=this->endPoints.at(row).omegaMatrix.size();k<arg1;k++)
            endPoints[row].omegaMatrix[j]<<0;
        }

        endPoints[row].omegaMatrix.resize(arg1);
        for (int j=oldMatrixSize;j<arg1;j++){
            endPoints[row].omegaMatrix[j].fill(0,arg1);
            endPoints[row].omegaMatrix[j][j]=1;
        }


        for (int j=0;j<oldMatrixSize;j++){
            for (int k=oldMatrixSize;k<arg1;k++)
            endPoints[row].omegaIndecies[j]<<-1;
        }

        endPoints[row].omegaIndecies.resize(arg1);
        for (int j=oldMatrixSize;j<arg1;j++){
            endPoints[row].omegaIndecies[j].fill(-1,arg1);
        }

    }
    ui->_1_tw_etaDefinition->setColumnCount(arg1);
    ui->_1_tw_etaDefinition->setRowCount(arg1);

    if(arg1==0)
        endPoints[row].randomEffect=false;

    endPoints[row].numETA=endPoints[row].omegaIndecies.size();

    this->refresh__1_tw_etaDefinition();
}

void MainWindow::on__1_tw_etaDefinition_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn)
{
    refresh__1_tw_etaDefinition();
}

void MainWindow::on__1_cb_iivEndPoint_currentIndexChanged(int index)
{
    for(int i=0;i<endPoints.size();i++)
        if (index==0){ //mean
            endPoints[i].iivEndpoint=-1;
        }else if(index==1){ //median
            endPoints[i].iivEndpoint=50;
        }else if(index==2){ //75%
            endPoints[i].iivEndpoint=75;
        }else if(index==3){ //25%
            endPoints[i].iivEndpoint=25;
        }

    refresh_1_cp_typicalDRPlot();
}


void MainWindow::on__5_pb_importSimRawRes_clicked()
{
    QStringList rawResFileNames=QFileDialog::getOpenFileNames(this,"Select raw_results files for simulation models.  The number of files imported must be either one or the same number as number of simulations.",ui->__lb_directory->text(),"*.csv");

    simRawResDatas.clear();
    simRawResHeaders.clear();
    if(rawResFileNames.size()==1){
        simRawResDatas<<csvWithHeaderReadToDouble(rawResFileNames.at(0));
        simRawResHeaders<<csvWithHeaderReadHeader(rawResFileNames.at(0));
    }else if(rawResFileNames.size()==this->rawResDatas.at(0).size()){
        for (int i=0;i<rawResFileNames.size();i++){
            simRawResDatas<<csvWithHeaderReadToDouble(rawResFileNames.at(i));
            simRawResHeaders<<csvWithHeaderReadHeader(rawResFileNames.at(i));
        }
    }else{
        QMessageBox warning;
        warning.setText("The number of imported raw results files must be the same as number of simulations or one (if all the simulaitons were generated from the same model with the same parameters).");
        warning.exec();
    }
    ui->_5_tw_trueModelTMED->setRowCount(simRawResDatas.size());
}

void MainWindow::on__5_pb_simTrueModel_clicked()
{
    QString trueModelFileName=QFileDialog::getOpenFileName(this, "Select files with true model indicies (always start with 0!  i.e.,  0,1,2,3,...).",ui->__lb_directory->text(),"*.csv");

    QVector<QVector<double> > tempDM;
    bool somethingWentWrong=false;
    bool mayHaveWrongFile=false;

    tempDM=csvReadToDouble(trueModelFileName);
    trueModels.clear();

    if(tempDM.size()==this->rawResDatas.at(0).size()&tempDM.at(0).size()==1){
        for (int i=0;i<tempDM.size();i++){
            trueModels<<(int)tempDM.at(i).at(0);

            if (trueModels.last()<0|trueModels.last()>=rawResDatas.size())
                somethingWentWrong=true;
        }


    }else if(tempDM.size()==0){
        somethingWentWrong=true;
    }else if(tempDM.at(0).size()==this->rawResDatas.at(0).size()&tempDM.size()==1){
        for (int i=0;i<tempDM.at(0).size();i++){
            trueModels<<(int)tempDM.at(0).at(i);

            if (trueModels.last()<0|trueModels.last()>=rawResDatas.size())
                somethingWentWrong=true;
        }

    }else{
        somethingWentWrong=true;
    }

    if(somethingWentWrong){
        QMessageBox warning;
        warning.setText("The file imported does not follow the recognizable format.  Make sure that it is a csv file with either one row or column.");
        warning.exec();
    }else{
        ui->_5_tw_trueModelTMED->setRowCount(trueModels.size());
        for (int i=0;i<trueModels.size();i++){
            ui->_5_tw_trueModelTMED->setItem(i,0,new QTableWidgetItem(QString::number(trueModels.at(i))));

            if (simRawResHeaders.size()>i)
                if (rawResHeaders.at(trueModels.at(i))!=simRawResHeaders.at(i))
                    mayHaveWrongFile=true;
        }
        QMessageBox warning;
        warning.setText("Some of the headers for the raw_results files for the simulation models do not match with the estimation models.  Please double check.");
        warning.exec();
    }


}

void MainWindow::on__4_tw_doseChoice_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn)
{
    refresh_4_tw_doseChoice();
}

void MainWindow::refresh_4_tw_doseChoice(){
    QPen tempPen;
    tempPen.setWidth(2);
    tempPen.setColor(uppsalaRed);

    ui->_4_cp_doseChoiceDistribution->clearGraphs();
    histogram(ui->_4_tw_doseChoice,ui->_4_cp_doseChoiceDistribution,ceil((double)ui->_4_tw_doseChoice->rowCount()/(double)10));
    ui->_4_cp_doseChoiceDistribution->graph(0)->setPen(tempPen);

    ui->_4_cp_doseChoiceDistribution->graph(0)->setBrush(this->uppsalaRedTrans);
    ui->_4_cp_doseChoiceDistribution->graph(0)->setChannelFillGraph(0);

    ui->_4_cp_doseChoiceDistribution->replot();
}

void MainWindow::on__1_sb_numSimulations_valueChanged(int arg1)
{
    for(int i=0;i<endPoints.size();i++)
        endPoints[i].numSample=arg1;

    refresh_1_cp_typicalDRPlot();
}

void MainWindow::contextMenuEvent(QContextMenuEvent *event){

}

void MainWindow::on__1_le_possibleDoses_textEdited(const QString &arg1)
{

}

void MainWindow::on__1_le_possibleDoses_editingFinished()
{

}

void MainWindow::on__4_pb_clearCash_clicked()
{
    this->cash_endpoints.clear();
}
