#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <endpoint.h>
#include <qcustomplot.h>
#include <QTableWidgetItem>

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);

    QVector<QVector<QVector<QVector<double> > > > rawResDatas;  //model data row column
    QVector<QVector<QVector<double> > > simRawResDatas;  //data row column
    QVector<QStringList> simRawResHeaders; //data row column
    QVector<QVector<double> > simulationResults;
    QVector<double> theoreticalMED;
    QVector<int> trueModels;

    QVector<QVector<double> > selectedDoses;
    QStringList dsProbabilitiesSL;
    QVector<double> dsProbabilities;

    QVector<QStringList> rawResHeaders;
    QVector<endPoint> endPoints;
    QVector<double> independentVariables;
    double targetValue;

    double targetUpperbound;
    double targetLowerbound;

    void refresh_0_tw_RawResults();
    void refresh_2_tw_weightDefinition();
    void refresh_2_tw_relativeWeight();
    QVector<QVector<double> > calculate_relativeWeight(int dataNumber);
    QVector<int> make_ModelSelection(int dataNumber);
    void refresh_0_cp_rawResHistoGram();

    void refresh_1_cp_typicalDRPlot();

    void refresh__1_tw_etaDefinition();

    void refresh_4_tw_doseChoice();

    QVector<double> calculate_PoS(int dataNumber);
    QVector<double> calculate_PoS_aux_modelSelect(QVector<QVector<QVector<double> > > endpoints);
    QVector<double> calculate_PoS_aux_modelAve(QVector<QVector<QVector<double> > > endpoints, QVector<QVector<double> > weights);


    void refresh_3_cp_PoS();
    int find_dose(QVector<double> PoS, double probability);

    QVector<double> find_dose(QVector<double> PoS, QVector<double> probabilities);
    QVector<double> find_dose(QVector<double> PoS);

    int OFV_index=20;
    int THETA_index=21;
    int ETA_index=22;

    QColor uppsalaRed=QColor(153, 0, 0,255);
    QColor uppsalaRedTrans=QColor(153, 0, 0,100);

    QVector<QVector<QVector<QVector<double> > > > cash_endpoints;


    ~MainWindow();

private slots:
    void on___pb_import_clicked();

    void on__0_sb_model_valueChanged(int arg1);

    void on__0_sb_data_valueChanged(int arg1);

    void on___mainToolBox_currentChanged(int index);

    void on__2_tw_weightDefinition_cellChanged(int row, int column);

    void on__2_cb_logScale_stateChanged(int arg1);

    void on__2_sb_dataNumber_valueChanged(int arg1);

    void on__0_tw_RawResults_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

    void mouseWheelDE();
    void mousePressDE();

    void mouseWheel_4_cp_doseDist();
    void mousePress_4_cp_doseDist();

    void saveModels();
    void loadModels();

    void on__1_le_possibleDoses_textChanged(const QString &arg1);

    void on__3_sl_doseSlider_valueChanged(int value);

    void on__3_cb_dataNumber_valueChanged(int arg1);

    void on__3_cb_aveORselect_currentIndexChanged(int index);

    void on__3_cb_indOFVone_currentIndexChanged(int index);

    void on__4_pb_startCalculation_clicked();


    void on__4_pb_compTruth_clicked();

    void on___pb_importFbF_clicked();

    void on__1_tw_endPoints_cellChanged(int row, int column);

    void on__m1_le_colNumOFV_textEdited(const QString &arg1);

    void on__m1_le_colNumTHETA1_textEdited(const QString &arg1);

    void on__1_tw_endPoints_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

    void on__1_tw_endPoints_itemChanged(QTableWidgetItem *item);

    void on__3_pb_calculatePoS_clicked();

    void on__3_le_lowerBound_editingFinished();

    void on__3_le_upperBound_editingFinished();

    void on__1_tw_etaDefinition_cellChanged(int row, int column);

    void on__1_cb_numEtas_valueChanged(int arg1);

    void on__1_tw_etaDefinition_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

    void on__1_cb_iivEndPoint_currentIndexChanged(int index);

    void on__5_pb_importSimRawRes_clicked();

    void on__5_pb_simTrueModel_clicked();

    void on__4_tw_doseChoice_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

    void on__1_sb_numSimulations_valueChanged(int arg1);


    void on__1_le_possibleDoses_textEdited(const QString &arg1);

    void on__1_le_possibleDoses_editingFinished();

    void on__4_pb_clearCash_clicked();

private:
    Ui::MainWindow *ui;

protected:
    void contextMenuEvent(QContextMenuEvent *event);
};

#endif // MAINWINDOW_H
