#include "randomng.h"
#include <QString>
#include <math.h>
#include <QDebug>

randomNG::randomNG()
{
}


double randomNG::normal(){
    double u= (double)qrand()/(double)RAND_MAX;
    double v= (double)qrand()/(double)RAND_MAX;
    double x=pow(-2*log(u)/log(exp(1)),0.5)*cos(2*M_PI*v);

    return x;
}

QVector<double> randomNG::multiNormal(QVector<QVector<double> > varCov){
    QVector<double> out, temp_out, r;

    double temp;
    QVector<bool> includeIndex;
    QVector<QVector<double> > newMatrix, tempMatrix;
    int okRowCol=0;

    includeIndex.resize(varCov.size());
    tempMatrix.clear();
    for (int i=0; i<varCov.size();i++){
        temp=0;
        for (int j=0; j<varCov.at(i).size();j++){
            temp=temp+pow(varCov.at(i).at(j),2.0);
        }
        if (temp>0|temp<0){
            includeIndex[i]=true;
            tempMatrix<<varCov.at(i);
            okRowCol++;
        }else{
            includeIndex[i]=false;
        }
    }

    newMatrix.clear();
    newMatrix.resize(okRowCol);
    for (int i=0; i<okRowCol; i++){
        newMatrix[i].clear();
        for (int j=0; j<tempMatrix.at(i).size(); j++){
            if (includeIndex.at(j))
                newMatrix[i]<<tempMatrix.at(i).at(j);
        }
    }


    QVector<QVector<double> > L=choleskyDecomposition(newMatrix);
    temp_out.resize(L.size());
    r.resize(L.size());


    for (int i=0; i<L.size();i++){
        temp=0;
        r[i]=normal();
        for(int j=0;j<i+1;j++){
            temp=temp+r.at(j)*L.at(i).at(j);
        }
        temp_out[i]=temp;
    }
    out.clear();
    for(int i=0;i<varCov.size();i++){
        if (includeIndex.at(i)){
            out<<temp_out.at(0);
            temp_out.pop_front();
        }else{
            out<<0.0;
        }
    }


    return out;
}


QVector<QVector<double> > randomNG::choleskyDecomposition(QVector<QVector<double> > A){

    QVector<QVector<double> > L;
    double lsqc, lodp;
    L.resize(A.size());

    for (int i=0; i<A.size();i++){
        L[i].resize(A.at(i).size());
        lsqc=0;
        for(int j=0;j<i;j++){
            lodp=0;
            for (int k=0; k<j;k++){
                lodp=lodp+L.at(i).at(k)*L.at(j).at(k);//pow(L.at(i).at(j),2.0);
            }
            L[i][j]=(A.at(i).at(j)-lodp)/L.at(j).at(j);
            L[j][i]=0;
            lsqc=lsqc+pow(L.at(i).at(j),2.0);
        }
       // qDebug()<<A.at(i).at(i)-lsqc;
        L[i][i]=pow(A.at(i).at(i)-lsqc, 0.50);
    }
   // qDebug()<<L;
    return L;
}
